--- old/usr/src/cmd/lofiadm/main.c	Mon Sep 16 15:02:46 2013
+++ new/usr/src/cmd/lofiadm/main.c	Mon Sep 16 15:02:46 2013
@@ -64,7 +64,7 @@
 #include <blowfish/blowfish_impl.h>
 
 static const char USAGE[] =
-	"Usage: %s -a file [ device ] "
+	"Usage: %s [-r] -a file [ device ] "
 	" [-c aes-128-cbc|aes-192-cbc|aes-256-cbc|des3-cbc|blowfish-cbc]"
 	" [-e] [-k keyfile] [-T [token]:[manuf]:[serial]:key]\n"
 	"       %s -d file | device\n"
@@ -364,10 +364,12 @@
  */
 static void
 add_mapping(int lfd, const char *devicename, const char *filename,
-    mech_alias_t *cipher, const char *rkey, size_t rksz)
+    mech_alias_t *cipher, const char *rkey, size_t rksz, boolean_t rdonly)
 {
 	struct lofi_ioctl li;
 
+	li.li_readonly = rdonly;
+
 	li.li_crypto_enabled = B_FALSE;
 	if (cipher != NULL) {
 		/* set up encryption for mapped file */
@@ -497,7 +499,7 @@
 	int	minor;
 	int	maxminor;
 	char	path[MAXPATHLEN];
-	char	options[MAXPATHLEN];
+	char	options[MAXPATHLEN] = { 0 };
 
 	li.li_minor = 0;
 	if (ioctl(fd, LOFI_GET_MAXMINOR, &li) == -1) {
@@ -517,6 +519,9 @@
 		}
 		(void) snprintf(path, sizeof (path), "/dev/%s/%d",
 		    LOFI_BLOCK_NAME, minor);
+
+		options[0] = '\0';
+
 		/*
 		 * Encrypted lofi and compressed lofi are mutually exclusive.
 		 */
@@ -526,7 +531,17 @@
 		else if (li.li_algorithm[0] != '\0')
 			(void) snprintf(options, sizeof (options),
 			    gettext("Compressed(%s)"), li.li_algorithm);
-		else
+		if (li.li_readonly) {
+			if (strlen(options) != 0) {
+				(void) strlcat(options, ",", sizeof (options));
+				(void) strlcat(options, "Readonly",
+				    sizeof (options));
+			} else {
+				(void) snprintf(options, sizeof (options),
+				    gettext("Readonly"));
+			}
+		}
+		if (strlen(options) == 0)
 			(void) snprintf(options, sizeof (options), "-");
 
 		(void) printf(FORMAT, path, li.li_filename, options);
@@ -1789,6 +1804,7 @@
 	const char *pname;
 	boolean_t errflag = B_FALSE;
 	boolean_t addflag = B_FALSE;
+	boolean_t rdflag = B_FALSE;
 	boolean_t deleteflag = B_FALSE;
 	boolean_t ephflag = B_FALSE;
 	boolean_t compressflag = B_FALSE;
@@ -1808,7 +1824,7 @@
 	(void) setlocale(LC_ALL, "");
 	(void) textdomain(TEXT_DOMAIN);
 
-	while ((c = getopt(argc, argv, "a:c:Cd:efk:o:s:T:U")) != EOF) {
+	while ((c = getopt(argc, argv, "a:c:Cd:efk:o:rs:T:U")) != EOF) {
 		switch (c) {
 		case 'a':
 			addflag = B_TRUE;
@@ -1863,6 +1879,9 @@
 			need_crypto = B_TRUE;
 			cipher_only = B_FALSE;	/* need to unset cipher_only */
 			break;
+		case 'r':
+			rdflag = B_TRUE;
+			break;
 		case 's':
 			segsize = convert_to_num(optarg);
 			if (segsize < DEV_BSIZE || !ISP2(segsize))
@@ -1893,6 +1912,7 @@
 	/* Check for mutually exclusive combinations of options */
 	if (errflag ||
 	    (addflag && deleteflag) ||
+	    (rdflag && !addflag) ||
 	    (!addflag && need_crypto) ||
 	    ((compressflag || uncompressflag) && (addflag || deleteflag)))
 		usage(pname);
@@ -2009,7 +2029,8 @@
 	 * Now to the real work.
 	 */
 	if (addflag)
-		add_mapping(lfd, devicename, filename, cipher, rkey, rksz);
+		add_mapping(lfd, devicename, filename, cipher, rkey, rksz,
+		    rdflag);
 	else if (compressflag)
 		lofi_compress(&lfd, filename, compress_index, segsize);
 	else if (uncompressflag)
